#  Copyright (c) 2019 Brian Schubert
#
#  This file is distributed under the MIT License. If a copy of the
#  MIT License was not distributed with this file, you can obtain one
#  at https://opensource.org/licenses/MIT.

from django.apps import AppConfig


class HTCPCPTeaConfig(AppConfig):
    name = 'django_htcpcp_tea'
    verbose_name = 'HTCPCP-TEA Server'
