{% extends "django_htcpcp_tea/base_error.html" %}

{% block error_title %}418 I'm a tea pot.{% endblock %}

{% block error_body %}
    <p>You're request code not be processed since the server is a teapot. This entity body MAY be short and stout.</p>
{% endblock %}
