{% extends "django_htcpcp_tea/base_error.html" %}

{% block error_title %}406 Not Acceptable{% endblock %}

{% block error_body %}
    <p>The operator of the coffee pot cannot comply with the requested additions.</p>

    {% if supported_additions %}
        <h2>Acceptable Additions</h2>
        {% include "django_htcpcp_tea/includes/additions.html" with additions=supported_additions %}
    {% else %}
        <p>This pot does not support beverage additions.</p>
    {% endif %}
{% endblock %}
