# Generated by Django 2.2.2 on 2019-06-20 12:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_htcpcp_tea', '0003_auto_20190620_1205'),
    ]

    operations = [
        migrations.AddField(
            model_name='addition',
            name='type',
            field=models.CharField(choices=[('MLK', 'Milk'), ('SYP', 'Syrup'), ('SWT', 'Sweetener'), ('SPC', 'Spice'), ('ACL', 'Alcohol'), ('SUG', 'Sugar'), ('OTR', 'Other')], default='OTR', max_length=3, verbose_name='Addition type'),
            preserve_default=False,
        ),
    ]
